/*
 * Component Name: pd.authz
 * File Name: azn_ent_cred_attrs.cpp
 *
 * IBM Confidential
 * Object Code Only Source Materials
 * 5747-SM3
 * (c) Copyright International Business Machines Corp. 2003
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>

#include <ctype.h>
#include <ogauthzn.h>
#include <azn_svc_protos.h>
#include <aznutils.h>


// this version
#define ENT_SVC_DEMO_VER "Tivoli Access Manager Entitlements Service Demo Plugin v1.0"
#define MAX_ATTR_LENGTH (2048)
#define LINELEN          (128)

#ifdef WIN32
#define STRCASECMP _stricmp
#define STRNCASECMP _strnicmp
#else
#define STRCASECMP strcasecmp
#define STRNCASECMP strncasecmp
#endif /* WIN32 */

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

#ifdef __cplusplus
extern "C" {
#endif


void attrlist_dump(azn_attrlist_h_t);
void check_status(char*, unsigned);
int readVar(const char*, char*, int);

/*
 * Ignore_unauth will be set to true if we should not attempt to update
 * the credentials of unauthenticated users.
 */
int ignore_unauth = FALSE;

/*
 * Dump_cred will be set to true if we should print out the contents of
 * the credential before asking for new attributes.
 */
int dump_cred = FALSE;



/******************************************************************************
 * Interface function definitions.                                            *
 ******************************************************************************/

/**
 * FUNCTION NAME
 *      azn_svc_initialize
 *
 * DESCRIPTION
 *      init the credattrs entitlements service
 *
 * ARGUMENTS
 *      [in] argc       The count of arguments to the service.
 *      [in] argv       The array of argument strings.
 *      [in] svcInit    List of initialization attributes for the service.
 *      [out] svcInfo   attr list ptr for attributes returned by the service.
 *
 * RETURN VALUE
 *      AZN_S_COMPLETE on success, error code on failure
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_initialize(const int                 argc,       /* in */
                   const char                **argv,     /* in */
                   const azn_attrlist_h_t    svcInit,    /* in */
                   azn_attrlist_h_t          *svcInfo    /* out */
                  )
{
    azn_status_t status = AZN_S_COMPLETE;
    azn_string_t svcid = NULL;
    azn_string_t azn_code_set = NULL;
    azn_string_t conf_file = NULL;
    azn_string_t domain = NULL;
    azn_boolean_t       freeAttrlist = FALSE;
    int i=0;

    if (svcInfo == NULL)
    {
        return(azn_util_errcode(AZN_S_SVC_ENT_INVALID_SVCINFO_HDL, 0));
    }

    if ((argc < 0) || (argc == 0 && argv != NULL))
    {
        return(azn_util_errcode(AZN_S_SVC_ENT_INVALID_ARG_COUNT, 0));
    }

    if ((argc > 0) && (argv == NULL))
    {
        return(azn_util_errcode(AZN_S_SVC_ENT_INVALID_ARG_COUNT, 0));
    }

    /* print the arguments */
    printf("%s\n\nazn_svc_initialize() Parameters:\n", ENT_SVC_DEMO_VER);

    for (i=0; i < argc; i++)
    {
        printf("param[%i] = %s\n", i, argv[i]);
        if (strcmp(argv[i], "-ignore_unauth") == 0)
        {
            printf("...ignoring unauthenticated users\n");
            ignore_unauth = TRUE;
        }
        else 
	if (strcmp(argv[i], "-dump_cred") == 0)
        {
            printf("...displaying input credentials\n");
            dump_cred = TRUE;
        }
    }

    // get my service id - must have
    status = azn_attrlist_get_entry_string_value(svcInit,
                                                 azn_svc_init_my_service_id,
                                                 AZN_C_INITIATOR_INDEX,
                                                 &svcid);
    if (status == AZN_S_COMPLETE || svcid != NULL)
    {
        printf("Entitilement Service Demo ID: %s\n", svcid);
        azn_release_string(&svcid);
    }
    else
    {
        return(azn_util_errcode(status, 0));
    }

    // get the conf file if there is one
    status = azn_attrlist_get_entry_string_value(svcInit,
                                                 azn_init_cfg_file,
                                                 AZN_C_INITIATOR_INDEX,
                                                 &conf_file);

    if (status == AZN_S_COMPLETE || conf_file != NULL)
    {
        printf("Entitilement Service Demo Configuration File: %s\n", conf_file);
        azn_release_string(&conf_file);
    }
    status = azn_attrlist_get_entry_string_value(svcInit,
                                                 azn_svc_init_code_set,
                                                 AZN_C_INITIATOR_INDEX,
                                                 &azn_code_set);

    if (status == AZN_S_COMPLETE || azn_code_set != NULL)
    {
        printf(" Entitilement Service Demo Codepage = %s\n", azn_code_set);
        azn_release_string(&azn_code_set);
    }

    // get the local domain this app is configured into
    // if the attribute is not set then assume NULL which means Default
    status = azn_attrlist_get_entry_string_value(svcInit,
                                                 azn_init_ssl_local_domain,
                                                 AZN_C_INITIATOR_INDEX,
                                                 &domain);

    if (status == AZN_S_COMPLETE || domain != NULL)
    {
        printf(" Entitlement Service Demo Domain = %s\n", domain);
        azn_release_string(&domain);
    }

    /* return the service version to the dispatcher */
    if (*svcInfo == AZN_C_INVALID_HANDLE)
    {
        azn_attrlist_create(svcInfo);
        freeAttrlist = TRUE;
    }
    // add this version info to svcInfo
    status = azn_attrlist_add_entry(*svcInfo,
                                    (const azn_string_t)azn_svc_version,
                                    (const azn_string_t)ENT_SVC_DEMO_VER);
    if (status != AZN_S_COMPLETE)
    {
        if (freeAttrlist)
        {
            azn_attrlist_delete(svcInfo);
        }

    }

    return(azn_util_errcode(AZN_S_COMPLETE, 0));
}


/**
 * FUNCTION NAME
 *      azn_svc_shutdown
 *
 * DESCRIPTION
 *      shutdown the ext_attr entitlements service.
 *      The initialization parameters are passed in
 *      again on shutdown but are ignored. No version info
 *      is returned.
 *
 * ARGUMENTS
 *      [in] argc       The count of arguments to the service.
 *      [in] argv       The array of argument strings.
 *      [in] svcInit    List of initialization attributes for the service.
 *      [out] svcInfo   attr list ptr for attributes returned by the service.
 *
 * RETURN VALUE
 *      AZN_S_COMPLETE on success, error code on failure
 */

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_shutdown(const int                 argc,       // in
                 const char                **argv,     // in
                 const azn_attrlist_h_t    svcInit,    // in
                 azn_attrlist_h_t          *svcInfo    // out
                )
{
    return(azn_util_errcode(AZN_S_COMPLETE, 0));
}


/**
 * FUNCTION NAME
 *      azn_svc_entitlement_get_entitlements
 *
 * DESCRIPTION
 *      Returns extended attribute information associated with
 *      the cred passed in
 *
 * ARGUMENTS
 *      [in]    creds           The credentials of the caller
 *      [in]    svc_id          The id of the entitlements service
 *      [in]    app_context     attribute list containing information
 *                              on which extended attributes to get
 *      [out]   entitlements    attribute list containing the extended
 *                              attributes
 *
 * RETURN VALUE
 *      AZN_S_COMPLETE on success, error code on failure
 *
 *
 */

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_entitlement_get_entitlements(
			    const azn_creds_h_t       creds,         /* in */
			    const azn_string_t        svc_id,        /* in */
			    const azn_attrlist_h_t    app_context,   /* in */
			    azn_attrlist_h_t          *entitlements) /* out */
{

    azn_status_t status = AZN_S_COMPLETE;
    unsigned num = 0;
    unsigned numval = 0;
    int      i = 0;
    int      j = 0;
    azn_attrlist_h_t cred_attrs = AZN_C_INVALID_HANDLE;
    azn_string_t attr_name = NULL;
    azn_string_t auth_type = NULL;
    azn_boolean_t free_enthdl = FALSE;
    azn_string_t attr_id = NULL;
    azn_boolean_t    done = FALSE;
    char             buf[MAX_ATTR_LENGTH];

    printf("\n%s\nazn_svc_entitlement_get_entitlements()\n\n", ENT_SVC_DEMO_VER);

    // check that we dont have an invalid handle
    if (entitlements == NULL)
    {
        return(azn_util_errcode(AZN_S_INVALID_ENTITLEMENTS_HDL, 0));
    }

    printf("Service ID: %s\n\n", svc_id);

    if (*entitlements == AZN_C_INVALID_HANDLE)
    {
        status = azn_attrlist_create(entitlements);
        if (status != AZN_S_COMPLETE)
        {
            return(status);
        }
        free_enthdl = TRUE;

    }
    if (ignore_unauth)
    {
        /* Check to see whether the user is unauthenticated, and leave
         * quickly if they are.
         */
        status = azn_creds_get_attr_value_string(creds,
                                                 AZN_C_INITIATOR_INDEX,
                                                 azn_cred_mech_id,
                                                 &auth_type);
        check_status((char*)"azn_creds_get_attr_value_string", status);
        if (status == AZN_S_COMPLETE && strcmp(auth_type, IV_UNAUTH) == 0)
        {
            azn_release_string(&auth_type);
            printf("Unauthenticated user, skipping adding extended attributes.\n");
            return(AZN_S_COMPLETE);
        }
    }

    if (dump_cred)
    {
        /* Display the attrlist for the principal */
        status = azn_creds_get_attrlist_for_subject(creds,
                                                    AZN_C_INITIATOR_INDEX, &cred_attrs);
        check_status((char*)"azn_creds_get_attrlist_for_subject", status);
        if (status == AZN_S_COMPLETE)
        {
            printf("Current credential attributes:\n");
            attrlist_dump(cred_attrs);
            azn_attrlist_delete(&cred_attrs);
        }
    }

    // if the app_context contians stuff then prompt to enter the attribute value
    // otherwise, prompt to enter attribute name and value.

    if (app_context != AZN_C_INVALID_HANDLE)
    {
        // look for azn_perminfo_rules_adi_request first
        attr_id = azn_perminfo_rules_adi_request;

        status = azn_attrlist_name_get_num(app_context, attr_id, &num);
        if ((status != AZN_S_COMPLETE) || ( num <=  0))
        {
            attr_id =  azn_ent_cred_attrs_attribute;
            num = 0;
            status = azn_attrlist_name_get_num(app_context, attr_id, &num);
        }
        if (status == AZN_S_COMPLETE && num > 0)
        {
            // app_context is valid and non-empty handle

            attrlist_dump(app_context);

            for (i = 0; i < num && !done ; i++)
            {
                status = azn_attrlist_get_entry_string_value(app_context,
                                                             attr_id,
                                                             i,
                                                             &attr_name);
                if (status != AZN_S_COMPLETE)
                {
                    break;
                }
                else
                {

                    buf[0]=(char)NULL;
                    printf("\nAttribute name = %s\n", attr_name);
                    if (!readVar("Enter the number of attribute values, or 0 to finish.",
                                 buf,
                                 LINELEN))
                    {
                        break;
                    }

                    numval = atoi(buf);
                    if (numval == 0)
                    {
                        break;
                    }
                    for (j = 0; j < numval ; j++)
                    {
                        buf[0]=(char)NULL;
                        if (!readVar("\nEnter an attribute value , or hit enter to finish.", buf, MAX_ATTR_LENGTH))
                        {

                            done = TRUE;
                            break;
                        }
                        if (strcmp(buf, "") == 0)
                        {
                            done = TRUE;
                            break;
                        }
                        status = azn_attrlist_add_entry(*entitlements,
                                                        (const azn_string_t) attr_name,
                                                        (const azn_string_t)buf);
                        check_status((char *)"azn_attrlist_add_entry", status);
                        if (status != AZN_S_COMPLETE)
                        {
                            done = TRUE;
                            break;
                        }
                    }
                }
                azn_release_string(&attr_name);

            }

            if (status != AZN_S_COMPLETE)
            {
                if (free_enthdl)
                {
                    azn_attrlist_delete(entitlements);
                }
            }
            else
            {
                attrlist_dump(*entitlements);
            }

            return(status);
        }
    }
    // app_context is invalid or empty handle
    done = FALSE;
    status = AZN_S_COMPLETE;
    while (! done && status == AZN_S_COMPLETE)
    {
        buf[0] = (char)NULL;
        if (! readVar("\nEnter the number of attribute name, or 0 to finish.",
                      buf,
                      LINELEN))
        {
            break;
        }
        num = atoi(buf);
        if (num == 0)
        {
            break;
        }
        for (i = 0; i < num && !done && status == AZN_S_COMPLETE; i++)
        {
            buf[0] = (char)NULL;
            if (!readVar("\nEnter an attribute name, or hit enter to finish.",
                         buf,
                         LINELEN))
            {
                break;
            }

            if (strcmp(buf, "") == 0)
            {
                break;
            }

            /* Save the attribute name */
            attr_name = strdup( buf);
            buf[0] = (char)NULL;
            if (!readVar("\nEnter the number of attribute values, or 0 to finish.",
                         buf,
                         LINELEN))
            {
                break;
            }

            numval = atoi(buf);
            if (numval == 0)
            {
                break;
            }
            for (j = 0; j < numval && status == AZN_S_COMPLETE; j++)
            {
                buf[0] = (char)NULL;
                if (!readVar("\nEnter an attribute value, or hit enter to finish.",
                             buf,
                             MAX_ATTR_LENGTH))
                {
                    done = TRUE;
                    break;
                }
                if (strcmp(buf, "") == 0)
                {
                    done = TRUE;
                    break;
                }
                status = azn_attrlist_add_entry(*entitlements,
                                                (const azn_string_t)attr_name,
                                                (const azn_string_t)buf);
                check_status((char *)"azn_attrlist_add_entry", status);
            }
            free (attr_name);
        }
        done = TRUE;
    }

    if (status == AZN_S_COMPLETE)
    {
        attrlist_dump(*entitlements);
    }
    else
    {
        if (free_enthdl)
        {
            azn_attrlist_delete(entitlements);
        }
    }
    return(status);

}

/*
 * check status and print a message.
 */
void
check_status(
            char *info,
            unsigned status
            )
{
    azn_string_t errstr = NULL;
    unsigned majerr, minerr;
    azn_status_t rc;

    if (status == AZN_S_COMPLETE)
        return;

    majerr = azn_error_major(status);
    minerr = azn_error_minor(status);

    printf("\n%s: ", info);

    rc = azn_error_get_string(status, &errstr);
    if (rc == AZN_S_COMPLETE && errstr)
    {
        printf("%s (0x%08x/0x%08x)\n\n", errstr, majerr, minerr);
        azn_release_string(&errstr);
    }
    else
        printf("API failure (0x%08x/0x%08x)\n\n", majerr, minerr);
}



/*
 * Prompt the user for a variable.
 */
int
readVar(
       const char *prompt,
       char *var,
       int len
       )
{
    char        *startpos;
    char        *endpos;
    char    buffer[MAX_ATTR_LENGTH];

    /* Display prompt */
    printf("%s [%s]: ", prompt, var);
    fflush(stdout);

    /* Get user input */
    if (fgets(buffer, len, stdin) == NULL)
        return(FALSE);

    buffer[len-1] = '\0';

    /* Skip leading white space */
    startpos = buffer;
    while (*startpos && isspace(*startpos))
    {
        startpos++;
    }

    /* If empty, return the current value */
    if (! *startpos)
    {
        return(TRUE);
    }

    /* Skip trailing white space */
    endpos = startpos + strlen(startpos) - 1;
    while (endpos != startpos && isspace(*endpos))
    {
        endpos--;
    }
    endpos[1] = '\0';

    /* check for end of exit marker */
    if (STRCASECMP(startpos, "exit") == 0)
        return(FALSE);

    /* Copy the user's input to return */
    strcpy(var, startpos);
    return(TRUE);
}


void attrlist_dump(azn_attrlist_h_t perminfo)
{
    azn_string_t* attr_names;
    int i ,j;
    unsigned int count;
    azn_status_t    aznStatus;
    azn_buffer_t buffer_value;
    azn_string_t str_value;
    azn_ulong_t                 ulong_value;

    aznStatus = azn_attrlist_get_names(perminfo, &attr_names);
    check_status((char *)"azn_attrlist_get_names", aznStatus);
    if (attr_names)
    {
        for (i = 0; attr_names[i] != NULL ;i++)
        {
            aznStatus = azn_attrlist_name_get_num(perminfo,
                                                  attr_names[i],  &count);
            check_status((char *)"azn_attrlist_name_get_num", aznStatus);

            printf("\n-----------------------------\n");
            printf("Attr Name = %s\n" , attr_names[i]);
            printf("Values are : \n");
            for (j=0;j<count;j++)
            {
                aznStatus = azn_attrlist_get_entry_string_value(perminfo,attr_names[i],j,&str_value);
                if (aznStatus == AZN_S_COMPLETE)
                {
                    printf("%s\n", str_value);
                    azn_release_string(&str_value);
                }
                else
                {
                    aznStatus = azn_attrlist_get_entry_buffer_value(perminfo,attr_names[i],j,&buffer_value);
                    if (aznStatus == AZN_S_COMPLETE)
                    {
                        printf("0x%08x\n" , buffer_value);
                        azn_release_buffer(&buffer_value);
                    }
                    else
                    {
                        aznStatus = azn_attrlist_get_entry_ulong_value(perminfo,attr_names[i],j,&ulong_value);
                        if (aznStatus == AZN_S_COMPLETE)
                            printf("%d\n" , ulong_value);
                    }

                }
            }
        }
    }
}


#ifdef __cplusplus
}
#endif

